"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _reactNative = require("react-native");

var _RTCRtpCapabilities = require("./RTCRtpCapabilities");

var _RTCRtpSendParameters = _interopRequireDefault(require("./RTCRtpSendParameters"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  WebRTCModule
} = _reactNative.NativeModules;

class RTCRtpSender {
  constructor(info) {
    _defineProperty(this, "_id", void 0);

    _defineProperty(this, "_track", null);

    _defineProperty(this, "_peerConnectionId", void 0);

    _defineProperty(this, "_rtpParameters", void 0);

    this._peerConnectionId = info.peerConnectionId;
    this._id = info.id;
    this._rtpParameters = info.rtpParameters;

    if (info.track) {
      this._track = info.track;
    }
  }

  async replaceTrack(track) {
    try {
      await WebRTCModule.senderReplaceTrack(this._peerConnectionId, this._id, track ? track.id : null);
    } catch (e) {
      return;
    }

    this._track = track;
  }

  static getCapabilities(kind) {
    if (kind === 'audio') {
      return _RTCRtpCapabilities.DEFAULT_AUDIO_CAPABILITIES;
    }

    if (!_RTCRtpCapabilities.senderCapabilities) {
      throw new Error('sender Capabilities are null');
    }

    return _RTCRtpCapabilities.senderCapabilities;
  }

  getParameters() {
    return this._rtpParameters;
  }

  setParameters(parameters) {
    return WebRTCModule.senderSetParameters(this._peerConnectionId, this._id, JSON.parse(JSON.stringify(parameters))) // This allows us to get rid of private "underscore properties"
    .then(newParameters => {
      this._rtpParameters = new _RTCRtpSendParameters.default(newParameters);
    });
  }

  get track() {
    return this._track;
  }

  get id() {
    return this._id;
  }

}

exports.default = RTCRtpSender;
//# sourceMappingURL=RTCRtpSender.js.map