function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { NativeModules } from 'react-native';
import { senderCapabilities, DEFAULT_AUDIO_CAPABILITIES } from './RTCRtpCapabilities';
import RTCRtpSendParameters from './RTCRtpSendParameters';
const {
  WebRTCModule
} = NativeModules;
export default class RTCRtpSender {
  constructor(info) {
    _defineProperty(this, "_id", void 0);

    _defineProperty(this, "_track", null);

    _defineProperty(this, "_peerConnectionId", void 0);

    _defineProperty(this, "_rtpParameters", void 0);

    this._peerConnectionId = info.peerConnectionId;
    this._id = info.id;
    this._rtpParameters = info.rtpParameters;

    if (info.track) {
      this._track = info.track;
    }
  }

  async replaceTrack(track) {
    try {
      await WebRTCModule.senderReplaceTrack(this._peerConnectionId, this._id, track ? track.id : null);
    } catch (e) {
      return;
    }

    this._track = track;
  }

  static getCapabilities(kind) {
    if (kind === 'audio') {
      return DEFAULT_AUDIO_CAPABILITIES;
    }

    if (!senderCapabilities) {
      throw new Error('sender Capabilities are null');
    }

    return senderCapabilities;
  }

  getParameters() {
    return this._rtpParameters;
  }

  setParameters(parameters) {
    return WebRTCModule.senderSetParameters(this._peerConnectionId, this._id, JSON.parse(JSON.stringify(parameters))) // This allows us to get rid of private "underscore properties"
    .then(newParameters => {
      this._rtpParameters = new RTCRtpSendParameters(newParameters);
    });
  }

  get track() {
    return this._track;
  }

  get id() {
    return this._id;
  }

}
//# sourceMappingURL=RTCRtpSender.js.map