function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { defineCustomEventTarget } from 'event-target-shim';
import { NativeModules } from 'react-native';
import { addListener, removeListener } from './EventEmitter';
import RTCErrorEvent from './RTCErrorEvent';
const {
  WebRTCModule
} = NativeModules;
const TRANSCEIVER_EVENTS = ['error'];
export default class RTCRtpTransceiver extends defineCustomEventTarget(...TRANSCEIVER_EVENTS) {
  constructor(args) {
    super();

    _defineProperty(this, "_peerConnectionId", void 0);

    _defineProperty(this, "_sender", void 0);

    _defineProperty(this, "_receiver", void 0);

    _defineProperty(this, "_id", void 0);

    _defineProperty(this, "_mid", null);

    _defineProperty(this, "_direction", void 0);

    _defineProperty(this, "_currentDirection", void 0);

    _defineProperty(this, "_stopped", void 0);

    this._peerConnectionId = args.peerConnectionId;
    this._id = args.id;
    this._mid = args.mid ? args.mid : null;
    this._direction = args.direction;
    this._currentDirection = args.currentDirection;
    this._stopped = args.isStopped;
    this._sender = args.sender;
    this._receiver = args.receiver;

    this._registerEvents();
  }

  get id() {
    return this._id;
  }

  get mid() {
    return this._mid;
  }

  get stopped() {
    return this._stopped;
  }

  get direction() {
    return this._direction;
  }

  set direction(val) {
    if (!['sendonly', 'recvonly', 'sendrecv', 'inactive'].includes(val)) {
      throw new TypeError('Invalid direction provided');
    }

    if (this._stopped) {
      throw Error('Transceiver Stopped');
    }

    if (this._direction === val) {
      return;
    }

    WebRTCModule.transceiverSetDirection(this._peerConnectionId, this.id, val);
    this._direction = val;
  }

  get currentDirection() {
    return this._currentDirection;
  }

  get sender() {
    return this._sender;
  }

  get receiver() {
    return this._receiver;
  }

  stop() {
    if (this._stopped) {
      return;
    }

    WebRTCModule.transceiverStop(this._peerConnectionId, this.id);
  }

  _registerEvents() {
    addListener(this, 'transceiverStopSuccessful', ev => {
      if (ev.peerConnectionId !== this._peerConnectionId || ev.transceiverId !== this._id) {
        return;
      }

      this._stopped = true;
      this._direction = 'stopped';
      this._currentDirection = 'stopped';
      this._mid = null;
      removeListener(this);
    });
    addListener(this, 'transceiverOnError', ev => {
      if (ev.info.peerConnectionId !== this._peerConnectionId || ev.info.transceiverId !== this._id) {
        return;
      }

      if (ev.func === 'stopTransceiver') {
        this._stopped = false;
      } else if (ev.func === 'setDirection') {
        this._direction = ev.info.oldDirection;
      } // @ts-ignore


      this.dispatchEvent(new RTCErrorEvent('error', ev.func, ev.message));
    });
  }

}
//# sourceMappingURL=RTCRtpTransceiver.js.map